/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <gen_define.h>

#include <j1939_j1939.h>
#include <gen_extern_decl.h>
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
CAN_HandleTypeDef hcan1;

/* USER CODE BEGIN PV */
CO_TIMER_T timer1;
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_CAN1_Init(void);
/* USER CODE BEGIN PFP */
static void errorHandler(UNSIGNED8 error);
static UNSIGNED8 claimAddress(BOOL_T valid, UNSIGNED8 ownClaimAddr);
static RET_T transmitPgn(UNSIGNED32  pgn);
static void commInd(CO_COMM_STATE_EVENT_T commEvent);
static void canInd(CO_CAN_STATE_T canState);
static void receivePgn(UNSIGNED32 pgn, UNSIGNED8 node, J1939_RECEIVE_STATE_T state);
/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_CAN1_Init();
  /* USER CODE BEGIN 2 */

	/* In this evaluation example
	 * the hardware configuration is generated by the CubeIDE.
	 * Normally this is done by codrvHardwareInit() */
	//j1939drvHardwareInit();

	if (j1939drvCanInit(250u) != RET_OK)  {
		errorHandler(1);
	}

	if (j1939drvTimerSetup(J1939_TIMER_INTERVAL) != RET_OK)  {
		errorHandler(2);
	}

	/* stack init */
	if (j1939StackInit() != RET_OK)  {
		errorHandler(3);
	}

	/* setup event registration */
	if (j1939EventRegister_COMM_EVENT(commInd) != RET_OK)  {
		errorHandler(5);
	}

	if (j1939EventRegister_CAN_STATE(canInd) != RET_OK)  {
		errorHandler(6);
	}

	if (j1939EventRegister_CLAIM_ADDRESS(claimAddress) != RET_OK)  {
		errorHandler(7);
	}

	if (j1939EventRegister_TRANSMIT_PGN(transmitPgn) != RET_OK)  {
		errorHandler(8);
	}

	if (j1939EventRegister_RECEIVE_PGN(receivePgn) != RET_OK)  {
		errorHandler(9);
	}


	if (j1939ChangeDestinationAddress(53248u, 22u) != RET_OK)  {
		errorHandler(10);
	}

	/* enable CAN communication */
	if (j1939drvCanEnable() != RET_OK)  {
		errorHandler(4);
	}

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
		while (j1939CommTask() == CO_TRUE)  {
			/* more messages to process
			 * call j1939CommTask() again */
		}
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage 
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 384;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Activate the Over-Drive mode 
  */
  if (HAL_PWREx_EnableOverDrive() != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_6) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief CAN1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_CAN1_Init(void)
{

  /* USER CODE BEGIN CAN1_Init 0 */

  /* USER CODE END CAN1_Init 0 */

  /* USER CODE BEGIN CAN1_Init 1 */

  /* USER CODE END CAN1_Init 1 */
  hcan1.Instance = CAN1;
  hcan1.Init.Prescaler = 16;
  hcan1.Init.Mode = CAN_MODE_NORMAL;
  hcan1.Init.SyncJumpWidth = CAN_SJW_1TQ;
  hcan1.Init.TimeSeg1 = CAN_BS1_1TQ;
  hcan1.Init.TimeSeg2 = CAN_BS2_1TQ;
  hcan1.Init.TimeTriggeredMode = DISABLE;
  hcan1.Init.AutoBusOff = DISABLE;
  hcan1.Init.AutoWakeUp = DISABLE;
  hcan1.Init.AutoRetransmission = DISABLE;
  hcan1.Init.ReceiveFifoLocked = DISABLE;
  hcan1.Init.TransmitFifoPriority = DISABLE;
  if (HAL_CAN_Init(&hcan1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN CAN1_Init 2 */

  /* USER CODE END CAN1_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOI_CLK_ENABLE();
  __HAL_RCC_GPIOJ_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOI, GPIO_PIN_15, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOJ, GPIO_PIN_1, GPIO_PIN_RESET);

  /*Configure GPIO pin : PI15 */
  GPIO_InitStruct.Pin = GPIO_PIN_15;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOI, &GPIO_InitStruct);

  /*Configure GPIO pin : PJ1 */
  GPIO_InitStruct.Pin = GPIO_PIN_1;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOJ, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */
/*********************************************************************/
/*
 * registered function for claim address
 */
static UNSIGNED8 claimAddress(
		BOOL_T valid,				/* claim address valid */
		UNSIGNED8	claimAddr		/* claim address */
	)
{
UNSIGNED8 newAddress = claimAddr;

	if (valid == CO_TRUE)  {
		printf("Claim Address %d is valid now...\n", claimAddr);
	} else {
		printf("claim indication old address: %d invalid, ", claimAddr);
		if (claimAddr < 220u)  {
			newAddress = claimAddr + 1u;
		} else {
			return(255u);
		}
		printf("try new claim Address %d\n", newAddress);
	}

	return(newAddress);
}


/*********************************************************************/
/*
 * registered function for receive PGNs
 * (Called if PGN was received)
 */
static void receivePgn(
		UNSIGNED32  pgn,
		UNSIGNED8	node,
		J1939_RECEIVE_STATE_T state
	)
{
	printf("PGN %d received from node %d - state %d\n", pgn, node, state);

	if (pgn == 65128u)  {
		printf("HydraulicTemperature: %d °C\n",
			j1939Slot67_toDisp(HydraulicTemperature));
		printf("HydraulicOilFilterRestrictionSwitch: %d\n",
			HydraulicOilFilterRestrictionSwitch);
		printf("WinchOilPressureSwitch: %d\n", WinchOilPressureSwitch);
		printf("HydraulicOilLevel: %f %%\n",
			j1939Slot44_toDisp(HydraulicOilLevel));
	}

	if (pgn == 65212u)  {
		printf("TotalCompressionBrakeDistance %f\n",
			j1939Slot36_toDisp(TotalCompressionBrakeDistance));
		printf("TripCompressionBrakeDistance %f\n",
			j1939Slot36_toDisp(TripCompressionBrakeDistance));
		printf("TripSeviceBrakeApplication %f\n",
			j1939Slot36_toDisp(TripSeviceBrakeApplication));
		printf("TripSeviceBrakeDistance %d\n", TripSeviceBrakeDistance);
	}
}


/*********************************************************************/
/*
 * registered function for transmit PGNs
 * (Called before PGN is transmitted)
 */
static RET_T transmitPgn(
		UNSIGNED32  pgn
	)
{
static UNSIGNED8		percent = 50;
static UNSIGNED8		temperature = 10;

	printf("PGN %d to transmit\n", pgn);

	if (pgn == 65262u)  {
		temperature++;
		if (temperature > 100)  {
			temperature = 10;
		}
		ect = j1939Slot67_toCAN(temperature); /* Engine Coolant Temperature */
		ft = j1939Slot67_toCAN(temperature);  /* Fuel Temperature */
		eol = j1939Slot68_toCAN(temperature); /* Engine Oil Temperature 1 */
		tol = j1939Slot68_toCAN(temperature); /* Turbo Oil Temperature */
		eit = j1939Slot67_toCAN(temperature); /* Engine Intercooler Temperature */
		eito = j1939Slot44_toCAN(temperature);/* Engine Intercooler Thermostat Opening */
	}

	if (pgn == 53248u)  {
		percent++;
		if (percent == 100)  {
			percent = 0;
		}
		ilbp = j1939Slot44_toCAN(percent);	/* Illumination Brightness */
		sbbp = j1939Slot44_toCAN(percent);	/* switch backlight Brightness */
		sibp = j1939Slot44_toCAN(percent);	/* switch indication Brightness */
//j1939SendCmdt(64956, 27);
//j1939SendBam(64956);
	}

	return(RET_OK);
}


/*********************************************************************/
/*
 * registered function for CAN events
 */
static void canInd(
		CO_CAN_STATE_T  canState
	)
{
	switch (canState)  {
	case CO_CAN_STATE_BUS_OFF:
		printf("CAN: Bus Off\n");
		break;
	case CO_CAN_STATE_BUS_ON:
		printf("CAN: Bus On\n");
		break;
	case CO_CAN_STATE_PASSIVE:
		printf("CAN: Passive\n");
		break;
	default:
		break;
	}
}


/*********************************************************************/
/*
 * registered function for Communication events
 */
static void commInd(
		CO_COMM_STATE_EVENT_T   commEvent
	)
{
	switch (commEvent)  {
	case CO_COMM_STATE_EVENT_CAN_OVERRUN:
		printf("COMM-Event: CAN Overrun\n");
		break;
	case CO_COMM_STATE_EVENT_REC_QUEUE_FULL:
		printf("COMM-Event: Rec Queue Full\n");
		break;
	case CO_COMM_STATE_EVENT_REC_QUEUE_OVERFLOW:
		printf("COMM-Event: Rec Queue Overflow\n");
		break;
	case CO_COMM_STATE_EVENT_REC_QUEUE_EMPTY:
		printf("COMM-Event: Rec Queue Empty\n");
		break;
	case CO_COMM_STATE_EVENT_TR_QUEUE_FULL:
		printf("COMM-Event: Tr Queue Full\n");
		break;
	case CO_COMM_STATE_EVENT_TR_QUEUE_OVERFLOW:
		printf("COMM-Event: Tr Queue Overflow\n");
		break;
	case CO_COMM_STATE_EVENT_TR_QUEUE_EMPTY:
		printf("COMM-Event: Tr Queue Empty\n");
		break;
	case CO_COMM_STATE_EVENT_DRV_ERROR:
		printf("COMM-Event: Internal Driver Error\n");
		break;
	default:
		printf("COMM-Event: %d\n", commEvent);
		break;
	}
}


/*********************************************************************/
static void errorHandler(
		UNSIGNED8 error
	)
{
	printf("exit with error: %u\n", error);

	while (1)  {
		/* do nothing */
	}
}
/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
