/*
* j1939_j1939.h - contains defines for all services
*
* Copyright (c) 2014-2020 emotas embedded communication GmbH
*
*-------------------------------------------------------------------
* $Id: j1939_j1939.h 33239 2020-08-24 12:21:46Z boe $
*-------------------------------------------------------------------
*
*
*/

/**
* \brief defines for all services
*
* \file j1939_j1939.h - contains defines for all services
*
* This header inludes defines for all services of the J1939 library.
* It can be included instead of header files of each service.
*
*/

#ifndef J1939_J1939_H
#define J1939_J1939_H 1

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define UNSIGNED24_T    UNSIGNED32

#include <co_datatype.h>
#include <co_cob.h>
#include <co_commtask.h>
#include <co_drv.h>
#include <co_timer.h>

#include <j1939_datatype.h>
#include <j1939_indication.h>


#include <j1939_slots.h>

#ifdef J1939_DYNAMIC_PGN
#include <j1939_dynpgn.h>
#endif /* J1939_DYNAMIC_PGN */

#if defined(J1939_DIAGNOSTIC_SUPPORTED) || defined(J1939_DIAGNOSTIC_TOOL_SUPPORTED)
#include <j1939_diagnostic.h>
#endif /*  J1939_DIAGNOSTIC_SUPPORTED */

#ifdef J1939_NMEA2000_SUPPORTED
# include <j1939_nmea2000.h>
#endif /* J1939_NMEA2000_SUPPORTED */

/* actual J1939 stack software version */
#ifdef CANOPEN_SUPPORTED
#include <co_canopen.h>
#endif /* CANOPEN_SUPPORTED */
#ifndef CO_STACK_VERSION
# define CO_STACK_VERSION	0x30401ul
#endif /* CO_STACK_VERSION */


/* SPN lock macros */
#ifndef CO_OS_LOCK_OD
# define CO_OS_LOCK_OD
#endif /* CO_OS_LOCK_OD */
#ifndef CO_OS_UNLOCK_OD
# define CO_OS_UNLOCK_OD
#endif /* CO_OS_LOCK_OD */


typedef struct {
	UNSIGNED16	structVersion;
	UNSIGNED16	recBufferCnt[1];
	UNSIGNED16	trBufferCnt[1];
	UNSIGNED16	cobCnt[1];
	UNSIGNED16	receivePgnCnt[1];
	UNSIGNED16	transmitPgnCnt[1];
} J1939_SERVICE_INIT_VAL_T;


/* function prototypes */

/**
* \brief j1939_stackInit - init of J1939 stack 
*
* This function is normally generated by the J1939 Device Designer
* and responsible for the intialization of the J1939 stack.
*
* \return RET_T
*/
EXTERN_DECL void j1939drvHardwareInit(void);

EXTERN_DECL RET_T j1939drvTimerSetup(UNSIGNED32 timerInterval);

EXTERN_DECL RET_T j1939drvCanInit(UNSIGNED16 bitRate);
EXTERN_DECL RET_T j1939drvCanEnable(void);
EXTERN_DECL RET_T j1939drvCanDisable(void);
EXTERN_DECL RET_T j1939drvCanDeinit(void);

EXTERN_DECL RET_T j1939StackInit(void);
EXTERN_DECL RET_T j1939StackInit_common(const J1939_SERVICE_INIT_VAL_T *initVars);
EXTERN_DECL RET_T j1939StackInit_line(const J1939_CONFIG_T *pConfig);

EXTERN_DECL BOOL_T j1939CommTask(void);

EXTERN_DECL RET_T j1939AddressClaimingStart(UNSIGNED8 newAddress);
EXTERN_DECL RET_T j1939SendPgn(UNSIGNED32 pgn);
EXTERN_DECL RET_T j1939SendPgnAddr(UNSIGNED32 pgn, UNSIGNED8 destNode);
EXTERN_DECL RET_T j1939SendCmdt(UNSIGNED32 pgn, UNSIGNED8 dstNode);
EXTERN_DECL RET_T j1939SendBam(UNSIGNED32 pgn);
EXTERN_DECL RET_T j1939RequestPgn(UNSIGNED32 pgn, UNSIGNED8 destination);

EXTERN_DECL RET_T j1939ChangeDestinationAddress(UNSIGNED32 pgn,
				UNSIGNED8 dstAddress);

EXTERN_DECL RET_T j1939ProprietaryTransmit_A(UNSIGNED8 destNode);
EXTERN_DECL RET_T j1939ProprietaryTransmit_A_Length(UNSIGNED8 destNode, UNSIGNED16  length);
EXTERN_DECL RET_T j1939ProprietaryTransmit_B(UNSIGNED32 pgn);

EXTERN_DECL RET_T j1939PgnDisable(DIRECTION_T direction, UNSIGNED32 pgn);
EXTERN_DECL RET_T j1939PgnEnable(DIRECTION_T direction, UNSIGNED32 pgn);

EXTERN_DECL CO_CAN_TIMESTAMP_T j1939PgnGetTimestamp(UNSIGNED32 pgn);

EXTERN_DECL RET_T j1939SpnGet_u32(UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED32 *pValue);
EXTERN_DECL RET_T j1939SpnGet_u16(UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED16 *pValue);
EXTERN_DECL RET_T j1939SpnGet_u8(UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED8 *pValue);
EXTERN_DECL RET_T j1939SpnGet_vs(UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, VIS_STRING *pValue, UNSIGNED16 maxBuf);

EXTERN_DECL RET_T j1939SpnPut_u32(UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED32 value);
EXTERN_DECL RET_T j1939SpnPut_u16(UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED16 value);
EXTERN_DECL RET_T j1939SpnPut_u8(UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED8 value);
EXTERN_DECL RET_T j1939SpnPut_vs(UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, const VIS_STRING value, UNSIGNED16 size);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* J1939_J1939_H */
